@@ -16,7 +16,7 @@
 package Tk::JFileDialog;
 
 use vars qw($VERSION $bummer $MAXWIDTH);
-our $VERSION = '2.30';
+our $VERSION = '2.31';
 our $MAXWIDTH = 60;  #AVG. CHARACTERS.
 
 require 5.002;
@@ -55,8 +55,27 @@ eval 'use Tk::Dialog; 1'  if ($dialog_widget !~ 'J');
 
 my $driveletter = '';
 my %lastPaths;
-my $homedir = $bummer ? $ENV{'HOMEDRIVE'} . $ENV{'HOMEPATH'} : $ENV{'HOME'};
-$homedir ||= $ENV{'LOGDIR'}  if ($ENV{'LOGDIR'});
+my $homedir;
+
+#FIGURE OUT "HOME" DIRECTORY/FOLDER:
+if ($bummer)
+{
+	#M$-WINDOWS (bummer!):
+	if (defined $ENV{'HOMEDRIVE'})
+	{
+		$homedir = $ENV{'HOMEDRIVE'};
+		$homedir .= $ENV{'HOMEPATH'}  if (defined $ENV{'HOMEPATH'});
+	}
+	unless (defined($homedir) && $homedir =~ /\S/)
+	{
+		$homedir = $ENV{'LOGDIR'}  if (defined $ENV{'LOGDIR'});
+	}
+}
+unless (defined $homedir && $homedir =~ /\S/)
+{
+	$homedir = $ENV{'HOME'}  if (defined $ENV{'HOME'});
+	$homedir = ''  unless (defined $homedir);
+}
 $homedir =~ s#[\/\\]$##;
 
 @Tk::JFileDialog::ISA = qw(Tk::Toplevel);
@@ -136,13 +155,13 @@ sub Populate
 			-FNameList			=> ['PASSIVE', undef, undef, undef],
 			-FPat			=> ['PASSIVE', undef, undef, ''],
 			-FPatList			=> ['PASSIVE', undef, undef, undef],
-			-FPatFilters	=> ['PASSIVE', undef, undef, undef],
+			-FPatFilters	=> ['PASSIVE', undef, undef, ''],
 			-FPatOnly	=> ['PASSIVE', undef, undef, 0],
 			-File			=> ['PASSIVE', undef, undef, ''],
 			-Geometry		=> ['PASSIVE', undef, undef, undef],
 			-Grab			=> ['PASSIVE', undef, undef, 1],
 			-Horiz		=> ['PASSIVE', undef, undef, 1],
-			-Path			=> ['PASSIVE', undef, undef, undef],
+			-Path			=> ['PASSIVE', undef, undef, ''],
 			-SelDir		=> ['PASSIVE', undef, undef, 0],
 			-SortButton		=> ['PASSIVE', undef, undef, 1],
 			-SortOrder		=> ['PASSIVE', undef, undef, 'Name'],
@@ -166,12 +185,12 @@ sub Populate
 			-Title		=> ['PASSIVE', undef, undef, 'Select File:'],
 			-EDlgTitle		=> ['PASSIVE', undef, undef, 'Incorrect entry or selection!'],
 			-History => ['PASSIVE', undef, undef, 20],
-			-HistFile => ['PASSIVE', undef, undef, undef],
-			-HistDeleteOk => ['PASSIVE', undef, undef, undef],
+			-HistFile => ['PASSIVE', undef, undef, ''],
+			-HistDeleteOk => ['PASSIVE', undef, undef, 0],
 			-HistUsePath => ['PASSIVE', undef, undef, 0],
 			-HistUsePathButton => ['PASSIVE', undef, undef, 0],
-			-PathFile => ['PASSIVE', undef, undef, undef],
-			-DefaultFile => ['PASSIVE', undef, undef, undef],
+			-PathFile => ['PASSIVE', undef, undef, ''],
+			-DefaultFile => ['PASSIVE', undef, undef, ''],
 			-QuickSelect => ['PASSIVE', undef, undef, 1],
 			-DestroyOnHide => ['PASSIVE', undef, undef, 0],
 			-noselecttext => ['PASSIVE', undef, undef, 0],
@@ -486,15 +505,10 @@ sub IsNum
 
 sub BuildListBox
 {
-	my ($self, $fvar, $flabel, $listvar,$hpack, $vpack) = @_;
+	my ($self, $fvar, $flabel, $listvar, $hpack, $vpack) = @_;
 
 	## Create the subframe
-	$self->{$fvar} = $self->{'DFFrame'}->Frame
-			->pack(-side => $self->{'Configure'}{'-Horiz'} ? $hpack : $vpack,
-			-anchor => 'center',
-			-padx => '4m',
-			-pady => '2m',
-	@bothFill, @expand);
+	$self->{$fvar} = $self->{'DFFrame'}->Frame;
 
 	## Create the label
 	$self->{$fvar}->Label(-text => $flabel)->pack(@topPack, @xfill);
@@ -503,8 +517,11 @@ sub BuildListBox
 	my $fbf = $self->{$fvar}->Frame->pack(@topPack, @bothFill, @expand);
 
 	## And the scrollbar and listbox in it
-	$self->{$listvar} = $fbf->Scrolled('Listbox', -scrollbars => 'se', @raised, 
-			-exportselection => 0)->pack(@leftPack, @expand, @bothFill);
+	$self->{$listvar} = $fbf->Scrolled('Listbox',
+			-scrollbars => 'se', 
+			@raised, 
+			-exportselection => 0
+	)->pack(@leftPack, @expand, @bothFill);
 
 	$self->{$listvar}->Subwidget('xscrollbar')->configure(-takefocus => 0);
 	$self->{$listvar}->Subwidget('yscrollbar')->configure(-takefocus => 0);
@@ -513,7 +530,17 @@ sub BuildListBox
 	$self->{$listvar}->bind('<Leave>', sub { $self->bind('<MouseWheel>', [ sub { Tk->break; }]) });
 
 	#NEXT LINE ADDED 20050416 TO PERMIT MULTIFILE SELECTIONS, THANKS TO Paul Falbe FOR THIS PATCH!
-	$self->{$listvar}->configure(-selectmode => $self->{'Configure'}{'-SelectMode'})  if ($listvar eq 'FileList');
+	$self->{$listvar}->configure(-selectmode => $self->{'Configure'}{'-SelectMode'})
+			if ($listvar eq 'FileList');
+
+	#MUST BUILD (FOR BINDINGS' SAKES) BUT ONLY PACK IF USER IS TO SEE IT!:
+	$self->{$fvar}->pack(
+			-side => $self->{'Configure'}{'-Horiz'} ? $hpack : $vpack,
+			-anchor => 'center',
+			-padx => '4m',
+			-pady => '2m',
+			@bothFill, @expand
+	) if ($self->{'Configure'}{"-Show$listvar"});
 }
 
 sub BindDir
@@ -602,13 +629,14 @@ sub BindFile
 	## A single click selects (highlights) the file:
 	$self->{'FileList'}->bind('<ButtonRelease-1>', sub
 	{
+		my $fl = shift;
 		if ($self->{'Configure'}{'-SelDir'} != 1)
 		{
 #			$self->{'Configure'}{'-File'} =
-#					$self->{'FileList'}->get($self->{'FileList'}->curselection);
+#					$fl->get($fl->curselection);
 			#PREV. CHGD. TO NEXT 20050416 TO PERMIT MULTIFILE SELECTIONS, THANKS TO Paul Falbe FOR THIS PATCH!
- 			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $self->{'FileList'}->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
-					$self->{'FileList'}->curselection;
+ 			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $fl->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
+					$fl->curselection;
 			$self->{'FileEntry'}->icursor('end');
 			$self->{'FileEntry'}->selectionRange(0,'end')
 					unless ($self->{'Configure'}{'-noselecttext'});
@@ -625,12 +653,15 @@ sub BindFile
 		## A single-click completes the selection:
 		$self->{'FileList'}->bind('<1>', sub
 		{
+			my $fl = shift;
 			if ($self->{'Configure'}{'-SelDir'} != 1)
 			{
-				my $f = $self->{'FileList'}->curselection;
-				return if !defined($f);
-				($self->{'Configure'}{'-File'} = $self->{'FileList'}->get($f)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
-				$self->{'OK'}->invoke;
+				my $f = $fl->curselection;
+				if (defined $f)
+				{
+					($self->{'Configure'}{'-File'} = $fl->get($f)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
+					$self->{'OK'}->invoke;
+				}
 			}
 		});
 	}
@@ -639,61 +670,65 @@ sub BindFile
 		## A double-click completes the selection:
 		$self->{'FileList'}->bind('<Double-ButtonRelease-1>', sub
 		{
+			my $fl = shift;
 			if ($self->{'Configure'}{'-SelDir'} != 1)
 			{
-				my $f = $self->{'FileList'}->curselection;
-				return if !defined($f);
-				$self->{'OK'}->invoke;
+				my $f = $fl->curselection;
+
+				$self->{'OK'}->invoke  if (defined $f);
 			}
 		});
 	}
 	$self->{'FileList'}->bind('<Return>', sub
 	{
+		my $fl = shift;
 		if ($self->{'Configure'}{'-SelDir'} != 1)
 		{
-			my $f = $self->{'FileList'}->index('active');
+			my $f = $fl->index('active');
 			return if !defined($f);
-#			$self->{'Configure'}{'-File'} = $self->{'FileList'}->get($f);
+#			$self->{'Configure'}{'-File'} = $fl->get($f);
 			#PREV. CHGD. TO NEXT 20050416 TO PERMIT MULTIFILE SELECTIONS, THANKS TO Paul Falbe FOR THIS PATCH!
- 			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $self->{'FileList'}->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
-					$self->{'FileList'}->curselection;
+ 			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $fl->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
+					$fl->curselection;
 			$self->{'OK'}->focus;
 		}
 	});
 
 	$self->{'FileList'}->bind('<space>', sub
 	{
+		my $fl = shift;
 		if ($self->{'Configure'}{'-SelDir'} != 1)
 		{
-			my $f = $self->{'FileList'}->index('active');
+			my $f = $fl->index('active');
 			return if !defined($f);
-#			$self->{'Configure'}{'-File'} = $self->{'FileList'}->get($f);
+#			$self->{'Configure'}{'-File'} = $fl->get($f);
 			#PREV. CHGD. TO NEXT 20050416 TO PERMIT MULTIFILE SELECTIONS, THANKS TO Paul Falbe FOR THIS PATCH!
-			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $self->{'FileList'}->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
-					$self->{'FileList'}->curselection;
+			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $fl->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
+					$fl->curselection;
 		}
 	});
 
 	$self->{'FileList'}->bind('<Alt-s>', sub
 	{
+		my $fl = shift;
 		if ($self->{'Configure'}{'-SelDir'} != 1)
 		{
-			my $f = $self->{'FileList'}->index('active');
+			my $f = $fl->index('active');
 			return if !defined($f);
 
 			if ($ENV{'jwtlistboxhack'})
 			{
-				$self->{'FileList'}->jwtSpaceSelect($f);
+				$fl->jwtSpaceSelect($f);
 			}
 			else
 			{
-				$self->{'FileList'}->BeginSelect($f);
+				$fl->BeginSelect($f);
 			}
-			($self->{'File'} = $self->{'FileList'}->get($f)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
-#			$self->{'Configure'}{'-File'} = $self->{'FileList'}->get($f);
+			($self->{'File'} = $fl->get($f)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
+#			$self->{'Configure'}{'-File'} = $fl->get($f);
 			#PREV. CHGD. TO NEXT 20050416 TO PERMIT MULTIFILE SELECTIONS, THANKS TO Paul Falbe FOR THIS PATCH!
- 			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $self->{'FileList'}->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
-					$self->{'FileList'}->curselection;
+ 			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $fl->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
+					$fl->curselection;
 			Tk->break;
 		}
 	});
@@ -833,6 +868,7 @@ sub BuildPathEntry
 			-maxwidth => defined($self->{'Configure'}{'-maxwidth'}) ? $self->{'Configure'}{'-maxwidth'} : $MAXWIDTH,
 	)->pack(@leftPack, @expand, @xfill);
 	$self->{$entry}->choices(\%{$self->{'Configure'}{'PathList'}})  if ($favcnt);
+
 	if ($self->{'Configure'}{'-SortButton'})
 	{
 		$self->{'SortxButton'} = $eFrame->Checkbutton( -variable => \$self->{'Configure'}{'-SortOrder'},
@@ -857,7 +893,6 @@ sub BuildPathEntry
 		}
 	});
 
-	my $whichlist = 'DirList';
 	$self->{$entry}->bind('<Tab>',sub 
 	{
 		my ($oldval,$currentval);
@@ -868,7 +903,7 @@ sub BuildPathEntry
 			my $restofsel = $oldval;
 			if ($self->{'Configure'}{'-ShowDirList'})
 			{
-				($_ = $self->{$whichlist}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
+				($_ = $self->{'DirList'}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
 				$restofsel .= $_ . '/'  if ($_ && $_ ne '.' && $_ ne '..' && $_ ne '/');
 			}
 			$self->{'Configure'}{$LabelVar} = $restofsel;
@@ -882,9 +917,9 @@ sub BuildPathEntry
 		if ($self->{'Configure'}{'-ShowDirList'}) {
 			my ($currentval);
 			$currentval = $lastPaths{$driveletter};
-			$self->{$whichlist}->UpDown(-1);
+			$self->{'DirList'}->UpDown(-1);
 			my ($restofsel) = $currentval;
-			($_ = $self->{$whichlist}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
+			($_ = $self->{'DirList'}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
 			if (m#^\/$#o)
 			{
 				$restofsel = $_;
@@ -914,9 +949,9 @@ sub BuildPathEntry
 		elsif ($self->{'Configure'}{'-ShowDirList'}) {
 			my ($currentval);
 			$currentval = $lastPaths{$driveletter};
-			$self->{$whichlist}->UpDown(1);
+			$self->{'DirList'}->UpDown(1);
 			my $restofsel = $currentval;
-			($_ = $self->{$whichlist}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
+			($_ = $self->{'DirList'}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
 			if (m#^\/$#o)
 			{
 				$restofsel = $_;
@@ -974,17 +1009,6 @@ sub BuildFileEntry
 		close TEMP;
 	}
 
-	$self->{"histToggleVal"} = (defined($self->{'Configure'}{'-HistUsePathButton'}) && $self->{'Configure'}{'-HistUsePathButton'} == 1) ? 1 : 0;
-	if ($self->{'Configure'}{'-HistUsePath'}  && $self->{'Configure'}{'-HistUsePath'} != 1)
-	{
-		my $pathLabel = $self->{'Configure'}{'-HistUsePath'};
-		$pathLabel = 'Keep Path'  if ($self->{'Configure'}{'-HistUsePath'} =~ /^\-?\d/);
-		$self->{"histToggle"} = $eFrame->Checkbutton(
-				-text   => $pathLabel,
-				-variable=> \$self->{"histToggleVal"}
-		)->pack(@rightPack);
-	}
-
 	my $altbindings = (!defined($self->{'Configure'}{'-SelDir'}) || $self->{'Configure'}{'-SelDir'} != 1)
 			? 'Right=NoSearch,Return=NonEmptyGo' : 'Right=NoSearch';
 	$self->{$entry} = $eFrame->JBrowseEntry(@sunken,
@@ -1009,7 +1033,7 @@ sub BuildFileEntry
 			-noselecttext => $self->{'Configure'}{'-noselecttext'},
 			-listrelief => 'flat',
 			-maxwidth => defined($self->{'Configure'}{'-maxwidth'}) ? $self->{'Configure'}{'-maxwidth'} : $MAXWIDTH,
-	)->pack(@rightPack, @expand, @xfill);
+	)->pack(@leftPack, @expand, @xfill);
 	if ($Tk::JBrowseEntry::VERSION >= 5.0)
 	{
 		$self->{$entry}->configure(-deletecmd => sub {
@@ -1021,7 +1045,16 @@ sub BuildFileEntry
 	}
 	$self->{$entry}->choices(\@{$self->{'Configure'}{'HistList'}})  if ($histcnt);
 
-	my $whichlist = 'FileList';
+	$self->{"histToggleVal"} = (defined($self->{'Configure'}{'-HistUsePathButton'}) && $self->{'Configure'}{'-HistUsePathButton'} == 1) ? 1 : 0;
+	if ($self->{'Configure'}{'-HistUsePath'}  && $self->{'Configure'}{'-HistUsePath'} != 1)
+	{
+		my $pathLabel = $self->{'Configure'}{'-HistUsePath'};
+		$pathLabel = 'Keep Path'  if ($self->{'Configure'}{'-HistUsePath'} =~ /^\-?\d/);
+		$self->{"histToggle"} = $eFrame->Checkbutton(
+				-text   => $pathLabel,
+				-variable=> \$self->{"histToggleVal"}
+		)->pack(@leftPack);
+	}
 
 	$self->{$entry}->bind('<Escape>',sub {
 		$self->{$entry}->Popdown  if ($self->{$entry}->{'popped'});
@@ -1031,8 +1064,8 @@ sub BuildFileEntry
 		}
 		else
 		{
-		 	$self->{'Configure'}{'-File'} = join ',', map { (my $f = $self->{$whichlist}->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
-					$self->{$whichlist}->curselection;
+		 	$self->{'Configure'}{'-File'} = join ',', map { (my $f = $self->{'FileList'}->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
+					$self->{'FileList'}->curselection;
 			$self->{$entry}->icursor('end');
 			$self->{$entry}->selectionRange(0,'end')
 					unless ($self->{'Configure'}{'-noselecttext'});
@@ -1050,11 +1083,11 @@ sub BuildFileEntry
 				$currentval = ''  unless ($currentval =~ m#\/#o);
 				$currentval =~ s#(.*\/)(.*)$#$1#;
 				my ($restofsel) = $currentval;
-				($_ = $self->{$whichlist}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
+				($_ = $self->{'FileList'}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
 				if ($_ && $_ ne '.' && $_ ne '..' && $_ ne '/' && $_ !~ /\,/o)   #IF ADDED 20010131.
 				{
-#					($_ = $self->{$whichlist}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
-					my ($sel) = $self->{$whichlist}->curselection;
+#					($_ = $self->{'FileList'}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
+					my ($sel) = $self->{'FileList'}->curselection;
 					$restofsel .= $_  if ($sel =~ /\d/o);
 				}
 				$self->{'Configure'}{$LabelVar} = $restofsel;
@@ -1070,18 +1103,18 @@ sub BuildFileEntry
 			if ($self->{'Configure'}{'-SelectMode'} =~ /(?:multiple|extended)/io)
 			{
 				$self->{'Configure'}{$LabelVar} = '';
-				my $f = $self->{$whichlist}->index('active');
+				my $f = $self->{'FileList'}->index('active');
 				if (defined $f)
 				{
-		 			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $self->{$whichlist}->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
-					$self->{$whichlist}->curselection;
+		 			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $self->{'FileList'}->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
+					$self->{'FileList'}->curselection;
 				}
-				$self->{$whichlist}->focus();
+				$self->{'FileList'}->focus();
 			}
 			else
 			{
-				$self->{$whichlist}->UpDown(-1);
-				($_ = $self->{$whichlist}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
+				$self->{'FileList'}->UpDown(-1);
+				($_ = $self->{'FileList'}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
 				$self->{'Configure'}{$LabelVar} = $_;
 				$self->{$entry}->icursor('end');
 				$self->{$entry}->selectionRange(0,'end')
@@ -1101,18 +1134,18 @@ sub BuildFileEntry
 			if ($self->{'Configure'}{'-SelectMode'} =~ /(?:multiple|extended)/io)
 			{
 				$self->{'Configure'}{$LabelVar} = '';
-				my $f = $self->{$whichlist}->index('active');
+				my $f = $self->{'FileList'}->index('active');
 				if (defined $f)
 				{
-		 			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $self->{$whichlist}->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
-					$self->{$whichlist}->curselection;
+		 			$self->{'Configure'}{'-File'} = join ',', map { (my $f = $self->{'FileList'}->get($_)) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o; $f }
+					$self->{'FileList'}->curselection;
 				}
-				$self->{$whichlist}->focus();
+				$self->{'FileList'}->focus();
 			}
 			else
 			{
-				$self->{$whichlist}->UpDown(1);
-				($_ = $self->{$whichlist}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
+				$self->{'FileList'}->UpDown(1);
+				($_ = $self->{'FileList'}->get('active')) =~ s/^\d\d\d\d\d\d\d\d \d\d\d\d //o;
 				$self->{'Configure'}{$LabelVar} = $_;
 				$self->{$entry}->icursor('end');
 
@@ -1135,20 +1168,11 @@ sub BuildListBoxes
 	$self->{'DFFrame'} = $self->Frame;
 	$self->{'DFFrame'}->pack(-before => $self->{'FEF'}, @topPack, @bothFill, @expand);
 	
-	## Build the file window before the directory window, even
-	## though the file window is below the directory window, we'll
-	## pack the directory window before.
-	if ($self->{'Configure'}{'-ShowDirList'})
-	{
-		&BuildListBox($self,'DirFrame',$self->{'Configure'}{'-DirLBCaption'},'DirList','left','top');
-		&BindDir($self);
-	}
+	&BuildListBox($self,'DirFrame',$self->{'Configure'}{'-DirLBCaption'},'DirList','left','top');
+	&BindDir($self);
 
-	if ($self->{'Configure'}{'-ShowFileList'}) {
-		&BuildListBox($self, 'FileFrame',$self->{'Configure'}{'-FileLBCaption'},'FileList','right','bottom');
-		## Set up the bindings for the file list
-		&BindFile($self);
-	}
+	&BuildListBox($self, 'FileFrame',$self->{'Configure'}{'-FileLBCaption'},'FileList','right','bottom');
+	&BindFile($self);
 }
 
 sub BuildFDWindow
@@ -1488,7 +1512,7 @@ sub RescanFiles
 		#WE'LL EITHER LOAD FILES FROM A USER-PROVIDED LIST -OR- FROM THE CURRENT PATH:
 		my $pathNative = $path;
 		utf8::downgrade($pathNative, 1)  if ($self->{'Configure'}{'-nonLatinFilenames'});
-		if ($self->{'Configure'}{'-FNameList'} && ref($self->{'Configure'}{'-FNameList'}))
+		if (defined($self->{'Configure'}{'-FNameList'}) && ref($self->{'Configure'}{'-FNameList'}))
 		{
 FILELOOP1:			foreach my $f (@{$self->{'Configure'}{'-FNameList'}})  #USER-PROVIDED LIST:
 			{
@@ -1756,7 +1780,7 @@ sub GetReturn
 				elsif ($self->{'Configure'}{'-Create'} <= 0 && !$self->_fidtest('-d', $fname))
 				{
 					## Put up no create allowed dialog
-					$self->{'Configure'}{'-EDlgText'} = "You must specify an existing folder.\n(\$fname not found)";
+					$self->{'Configure'}{'-EDlgText'} = "You must specify an existing folder.\n($fname not found)";
 					eval "\$self->${dialog_widget}(-title => \$self->{'Configure'}{'-EDlgTitle'},
 							-text => \$self->{'Configure'}{'-EDlgText'},
 							-bitmap => 'error'
@@ -1783,8 +1807,14 @@ sub GetReturn
 				}
 				my $filenames = '';
 				my $errnames = '';
-				(my $protected = $self->{'Configure'}{'-FPatFilters'}) =~ s#\\\|#\x04#g;
-				my @filters = split (/\|/o, $protected);
+				my $protected = defined ($self->{'Configure'}{'-FPatFilters'})
+						? $self->{'Configure'}{'-FPatFilters'} : '';
+				my @filters = ();
+				if ($protected)
+				{
+					$protected =~ s#\\\|#\x04#g;
+					@filters = split (/\|/o, $protected);
+				}
 				@filters = ('')  unless ($#filters >= 0);
 				my @filters_rgx = ();
 				foreach my $f (@filters)
@@ -1913,7 +1943,7 @@ FILELOOP3:				foreach my $f (@filelist)   #ADD FULL PATHS TO ANY THAT ARE MISSIN
 				elsif ($self->{'Configure'}{'-Create'} <= 0 && !$self->_fidtest('-d', $fname))
 				{
 					## Put up no create allowed dialog
-					$self->{'Configure'}{'-EDlgText'} = "You must specify an existing directory.\n(\$fname not found)";
+					$self->{'Configure'}{'-EDlgText'} = "You must specify an existing directory.\n($fname not found)";
 					eval "\$self->${dialog_widget}(-title => \$self->{'Configure'}{'-EDlgTitle'},
 							-text => \$self->{'Configure'}{'-EDlgText'},
 							-bitmap => 'error'
@@ -2026,7 +2056,7 @@ FILELOOP4:				foreach my $f (@filelist)
 			if ($self->{'Configure'}{'-Create'} <= 0 && !$self->_fidtest('-d', $fname))
 			{
 				## Put up no create allowed dialog
-				$self->{'Configure'}{'-EDlgText'} = "You must specify an existing directory.\n(\$fname not found)";
+				$self->{'Configure'}{'-EDlgText'} = "You must specify an existing directory.\n($fname not found)";
 				eval "\$self->${dialog_widget}(-title => \$self->{'Configure'}{'-EDlgTitle'},
 						-text => \$self->{'Configure'}{'-EDlgText'},
 						-bitmap => 'error'
@@ -2140,6 +2170,8 @@ sub SortFiles
 
 sub _fidtest   #ADDED (v2.11) TO ADDRESS BUG# 128958: FILE TESTS FAIL ON *nix FILESYSTEMS IF PERL CONVERTS STRING TO UTF8?!
 {
+	return 0  unless (defined $_[2]);
+
 	my $strNative = $_[2];
 	utf8::downgrade($strNative, 1)  if ($_[0]->{'Configure'}{'-nonLatinFilenames'});  #LINUX STILL DOEN'T HANDLE UNICODE FILENAMES?!
 	my $s = "$_[1] \$strNative";
@@ -2215,11 +2247,11 @@ required.
 		-Create => 0
 	);
 
-	$LoadDialog->configure(-FPat => '*pl',
+	$LoadDialog->configure(-FPat => '*.pl',
 		-ShowAll => 0
 	);
 
-	$main->Entry(-textvariable => \$fname)
+	$main->Entry(-textvariable => \$fname, -width => 30)
 		->pack(-expand => 1,
 		-fill => 'x'
 	)->pack;
@@ -2228,7 +2260,7 @@ required.
 		-command => sub {
 			$fname = $LoadDialog->Show(-Horiz => $Horiz);
 			if (!defined($fname)) {
-				$fname = "Fine,Cancel, but no ShowDirList anymore!!!";
+				$fname = "Fine,Cancel, but no Dir. List anymore!!!";
 				$LoadDialog->configure(-ShowDirList => 0);
 			}
 		}
@@ -2481,7 +2513,7 @@ Default:  I<1>.
 =item B<-SelDir>
 
 If 1 or 2, enables selection of a directory rather than a file, and disables 
-theactions of the File List Box. Setting to 2 allows selection of either a 
+the actions of the File List Box. Setting to 2 allows selection of either a 
 file OR a directory.  If -1, the directory listbox, etc. are disabled and the 
 user is forced to select file(s) from the initially-specified path.  
 
@@ -2504,13 +2536,13 @@ Default: I<'browse'> (selecting only a single file from the list allowed).
 =item B<-SelHook>
 
 SelHook is configured with a reference to a routine that will be called when 
-a fileis chosen. The file is called with a sole parameter of the full path and 
-file nameof the file chosen. If the Create flag is disabled (and the user is 
-not allowedto specify new files), the file will be known to exist at the time 
-that SelHook is called. Note that SelHook will also be called with directories 
-if the SelDir Flag is enabled, and that the JFileDialog box will still be 
-displayed.  The JFileDialog box should B<not> be destroyed from within the 
-SelHook routine, although it may generally be configured.
+a fileis chosen. The function is called with a sole parameter of the full path 
+and file name of the file chosen. If the Create flag is disabled (and the user 
+is not allowedto specify new files), the file will be known to exist at the 
+time that SelHook is called. Note that SelHook will also be called with 
+directories if the SelDir Flag is enabled, and that the JFileDialog box will 
+still be displayed.  The JFileDialog box should B<not> be destroyed from within 
+the SelHook routine, although it may generally be configured.
 
 SelHook routines return 0 to reject the selection and allow the user to 
 reselect, and any other value to accept the selection. If a SelHook routine 
@@ -2675,7 +2707,7 @@ DEPRECIATED (now ignored)! - The message of the Error Dialog Box.
 The variables $path, $file, and $filename
 (the full path and filename of the selected file) are available.  
 
-Default:  I<"You must specify an existing file.\n(\$filename not found)">.
+Default:  I<"You must specify an existing file.\n($filename not found)">.
 
 =item B<-EDlgTitle>
 
